@@ -111,7 +111,7 @@ template<typename T>
 inline void v_zero(T *const BQ_R__ vec,
                    const int count)
 {
-    const T value = T(0);
+    const T value = T();
     for (int i = 0; i < count; ++i) {
         vec[i] = value;
     }
@@ -597,9 +597,9 @@ inline void v_increment(T *const BQ_R__ srcdst,
  *
  * Caller guarantees that \arg src and \arg srcdst are non-overlapping.
  */
-template<typename T>
+template<typename T, typename S>
 inline void v_multiply(T *const BQ_R__ srcdst,
-                       const T *const BQ_R__ src,
+                       const S *const BQ_R__ src,
                        const int count)
 {
     for (int i = 0; i < count; ++i) {
@@ -625,7 +625,7 @@ inline void v_multiply(double *const BQ_R__ srcdst,
 #endif // HAVE_IPP
 
 /**
- * v_multiply
+ * v_multiply_to
  *
  * Multiply the corresponding elements of the vectors \arg src1 and
  * \arg src2, both of length arg \count, and write the results into
@@ -634,10 +634,10 @@ inline void v_multiply(double *const BQ_R__ srcdst,
  * Caller guarantees that \arg src1, \arg src2 and \arg dst are
  * non-overlapping.
  */
-template<typename T>
+template<typename T, typename S>
 inline void v_multiply_to(T *const BQ_R__ dst,
                           const T *const BQ_R__ src1,
-                          const T *const BQ_R__ src2,
+                          const S *const BQ_R__ src2,
                           const int count)
 {
     for (int i = 0; i < count; ++i) {
@@ -700,6 +700,46 @@ inline void v_divide(double *const BQ_R__ srcdst,
 }
 #endif // HAVE_IPP
 
+/**
+ * v_divide_to
+ *
+ * Divide the corresponding elements of the vectors \arg src1 by those
+ * in \arg src2, both of length arg \count, and write the results into
+ * \arg dst.
+ *
+ * Caller guarantees that \arg src1, \arg src2 and \arg dst are
+ * non-overlapping.
+ */
+template<typename T, typename S>
+inline void v_divide_to(T *const BQ_R__ dst,
+                          const T *const BQ_R__ src1,
+                          const S *const BQ_R__ src2,
+                          const int count)
+{
+    for (int i = 0; i < count; ++i) {
+        dst[i] = src1[i] / src2[i];
+    }
+}
+
+#if defined HAVE_IPP 
+template<>
+inline void v_divide_to(float *const BQ_R__ dst,
+                          const float *const BQ_R__ src1,
+                          const float *const BQ_R__ src2,
+                          const int count)
+{
+    ippsDiv_32f(src1, src2, dst, count);
+}    
+template<>
+inline void v_divide_to(double *const BQ_R__ dst,
+                          const double *const BQ_R__ src1,
+                          const double *const BQ_R__ src2,
+                          const int count)
+{
+    ippsDiv_64f(src1, src2, dst, count);
+}
+#endif // HAVE_IPP
+
 /**
  * v_multiply_and_add
  *
@@ -760,9 +800,9 @@ inline T v_sum(const T *const BQ_R__ src,
 /**
  * v_multiply_and_sum
  *
- * Multiply the corresponding elements of the vectors \arg src1 and
- * \arg src2, both of length arg \count, sum the results, and return
- * the sum as a scalar value.
+ * Vector dot-product. Multiply the corresponding elements of the
+ * vectors \arg src1 and \arg src2, both of length arg \count, sum the
+ * results, and return the sum as a scalar value.
  *
  * Caller guarantees that \arg src1 and \arg src2 are non-overlapping.
  */
@@ -778,6 +818,46 @@ inline T v_multiply_and_sum(const T *const BQ_R__ src1,
     return result;
 }
 
+#if defined HAVE_IPP
+template<>
+inline float v_multiply_and_sum(const float *const BQ_R__ src1,
+                                const float *const BQ_R__ src2,
+                                const int count)
+{
+    float dp = 0.f;
+    ippsDotProd_32f(src1, src2, count, &dp);
+    return dp;
+}
+template<>
+inline double v_multiply_and_sum(const double *const BQ_R__ src1,
+                                 const double *const BQ_R__ src2,
+                                 const int count)
+{
+    double dp = 0.0;
+    ippsDotProd_64f(src1, src2, count, &dp);
+    return dp;
+}
+#elif defined HAVE_VDSP
+template<>
+inline float v_multiply_and_sum(const float *const BQ_R__ src1,
+                                const float *const BQ_R__ src2,
+                                const int count)
+{
+    float dp = 0.f;
+    vDSP_dotpr(src1, 1, src2, 1, &dp, count);
+    return dp;
+}
+template<>
+inline double v_multiply_and_sum(const double *const BQ_R__ src1,
+                                 const double *const BQ_R__ src2,
+                                 const int count)
+{
+    double dp = 0.0;
+    vDSP_dotprD(src1, 1, src2, 1, &dp, count);
+    return dp;
+}
+#endif
+
 /**
  * v_log
  *
@@ -1129,6 +1209,60 @@ inline void v_fftshift(T *const BQ_R__ vec,
     }
 }
 
+/**
+ * v_min
+ *
+ * Return the minimum of the values in the vector \arg vec, of length
+ * \arg count. For an empty vector, return 0.0.
+ */
+template<typename T>
+inline T v_min(const T *const BQ_R__ vec, const int count)
+{
+    if (count == 0) return T(0);
+    T min = vec[0];
+    for (int i = 1; i < count; ++i) {
+        T v = vec[i];
+        if (v < min) min = v;
+    }
+    return min;
+}
+
+/**
+ * v_max
+ *
+ * Return the maximum of the values in the vector \arg vec, of length
+ * \arg count. For an empty vector, return 0.0.
+ */
+template<typename T>
+inline T v_max(const T *const BQ_R__ vec, const int count)
+{
+    if (count == 0) return T(0);
+    T max = vec[0];
+    for (int i = 1; i < count; ++i) {
+        T v = vec[i];
+        if (v > max) max = v;
+    }
+    return max;
+}
+
+/**
+ * v_rms
+ *
+ * Return the RMS value of the values in the vector \arg vec, of
+ * length \arg count. For an empty vector, return 0.0.
+ */
+template<typename T>
+inline T v_rms(const T *const BQ_R__ vec, const int count)
+{
+    T t = T(0);
+    if (count == 0) return t;
+    for (int i = 0; i < count; ++i) {
+        t += vec[i] * vec[i];
+    }
+    t /= T(count);
+    return sqrt(t);
+}
+
 /**
  * v_mean
  *
